from django.apps import apps

from django_paddle_billing import settings as app_settings


def get_account_model():
    app, model = app_settings.PADDLE_ACCOUNT_MODEL.split(".")
    return apps.get_model(app, model, require_ready=False)



# Generated by Django 5.0.1 on 2024-01-24 11:46

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Account",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("user", models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



from django.conf import settings
from django_json_widget.widgets import JSONEditorWidget

CONFIG_DEFAULTS = {
    "PADDLE_API_TOKEN": "",
    "PADDLE_CLIENT_TOKEN": "",
    "PADDLE_SECRET_KEY": "",
    "PADDLE_API_URL": "https://sandbox-api.paddle.com",
    "PADDLE_IP_REQUEST_HEADER": "HTTP_X_FORWARDED_FOR",
    "PADDLE_IPS": ["34.232.58.13", "34.195.105.136", "34.237.3.244", "35.155.119.135", "52.11.166.252", "34.212.5.7"],
    "PADDLE_SANDBOX_IPS": [
        "34.194.127.46",
        "54.234.237.108",
        "3.208.120.145",
        "44.226.236.210",
        "44.241.183.62",
        "100.20.172.113",
    ],
    "PADDLE_SANDBOX": False,
    "PADDLE_ACCOUNT_MODEL": settings.AUTH_USER_MODEL,
    "PADDLE_ACCOUNT_LINK_BY_CUSTOMER": False,
    "ADMIN_READONLY": True,
    "ADMIN_JSON_EDITOR_WIDGET": JSONEditorWidget,
}


def get_config(settings_name=None):
    if settings_name is None:
        settings_name = "PADDLE_BILLING"

    return {**CONFIG_DEFAULTS, **getattr(settings, settings_name, {})}


settings = get_config()



import logging
from typing import Iterator, TypeVar

from apiclient import HeaderAuthentication
from django.conf import settings as django_settings
from django.contrib.auth import get_user_model
from django.db import models
from django.dispatch import receiver
from paddle_billing_client.client import PaddleApiClient
from paddle_billing_client.models import (
    address,
    business,
    customer,
    discount,
    price,
    product,
    subscription,
    transaction,
)
from paddle_billing_client.pagination import paginate

from django_paddle_billing import settings, signals
from django_paddle_billing.encoders import PrettyJSONEncoder
from django_paddle_billing.exceptions import DjangoPaddleBillingError
from django_paddle_billing.utils import get_account_model

logger = logging.getLogger(__name__)

paddle_client = PaddleApiClient(
    base_url=settings.PADDLE_API_URL,
    authentication_method=HeaderAuthentication(token=settings.PADDLE_API_TOKEN),
)

UserModel = get_user_model()

T = TypeVar("T", bound="PaddleBaseModel")


class PaddleBaseModel(models.Model):
    created_at = models.DateTimeField(auto_now_add=True, editable=False)
    updated_at = models.DateTimeField(auto_now=True, editable=False)
    occurred_at = models.DateTimeField(null=True, blank=True)

    class Meta:
        abstract = True

    def validate_occurred_at(self, occurred_at) -> bool:
        # Check if occurred_at is later than the current one
        if occurred_at is not None and self.occurred_at is not None and occurred_at < self.occurred_at:
            logger.info(
                f"{self.__class__.__name__}: The event is invalid, occurred_at is earlier"
                " than the current one - SKIP UPDATE"
            )
            return False
        return True

    @classmethod
    def update_or_create(cls: type[T], query, defaults, occurred_at=None) -> tuple[T, bool]:
        created = False
        try:
            instance = cls.objects.get(**query)
            valid = instance.validate_occurred_at(occurred_at)
            if not valid:
                return instance, created

        except cls.DoesNotExist:
            instance = cls(**query)
            created = True

        for k, v in defaults.items():
            setattr(instance, k, v)

        if occurred_at is not None:
            instance.occurred_at = occurred_at

        instance.save()

        return instance, created


class Product(PaddleBaseModel):
    id = models.CharField(max_length=50, primary_key=True)
    data = models.JSONField(null=True, blank=True, encoder=PrettyJSONEncoder)
    custom_data = models.JSONField(null=True, blank=True, encoder=PrettyJSONEncoder)
    name = models.CharField(max_length=255)
    status = models.CharField(max_length=10, choices=[("active", "Active"), ("archived", "Archived")])

    class Meta:
        pass

    def __str__(self) -> str:
        return f"{self.pk} - {self.name}"

    def get_data(self) -> product.Product | None:
        if self.data is None:
            return None
        return product.Product.model_validate(self.data)

    @classmethod
    def api_list_products(cls) -> product.ProductsResponse:
        return paddle_client.list_products()

    @classmethod
    def api_list_products_generator(cls, **kwargs) -> Iterator[product.ProductsResponse]:
        yield from paginate(paddle_client.list_products, query_params=product.ProductQueryParams(**kwargs))

    @classmethod
    def from_paddle_data(cls, data, occurred_at=None) -> tuple["Product | None", bool, Exception | None]:
        try:
            _product, created = cls.update_or_create(
                query={"pk": data.id},
                defaults={
                    "name": data.name,
                    "status": data.status,
                    "data": data.dict(),
                    "custom_data": data.custom_data,
                },
                occurred_at=occurred_at,
            )
            return _product, created, None
        except Exception as e:
            return None, False, e

    @classmethod
    def sync_from_paddle(cls) -> tuple[int, int]:
        logger.info("Sync Products from Paddle")
        created = 0
        updated = 0
        error = 0
        for products in cls.api_list_products_generator():
            for product_data in products.data:
                _product, _created, _error = cls.from_paddle_data(product_data)
                if _error:
                    error += 1
                elif _created:
                    created += 1
                else:
                    updated += 1
            logger.info("Product sync progress --- synced: %s, created: %s, errors: %s", updated, created, error)
        return created, updated


class Price(PaddleBaseModel):
    id = models.CharField(max_length=50, primary_key=True)
    data = models.JSONField(null=True, blank=True, encoder=PrettyJSONEncoder)
    custom_data = models.JSONField(null=True, blank=True, encoder=PrettyJSONEncoder)
    product = models.ForeignKey(Product, on_delete=models.CASCADE, related_name="prices")

    class Meta:
        pass

    def __str__(self) -> str:
        return str(self.pk)

    def get_data(self) -> price.Price | None:
        if self.data is None:
            return None
        return price.Price.model_validate(self.data)

    @classmethod
    def api_list_prices(cls) -> price.PricesResponse:
        return paddle_client.list_prices()

    @classmethod
    def api_list_prices_generator(cls, **kwargs) -> Iterator[price.PricesResponse]:
        yield from paginate(paddle_client.list_prices, **kwargs)

    @classmethod
    def from_paddle_data(cls, data, occurred_at=None) -> tuple["Price | None", bool, Exception | None]:
        try:
            _price, created = cls.update_or_create(
                query={"pk": data.id},
                defaults={
                    "product_id": data.product_id,
                    "data": data.dict(),
                    "custom_data": data.custom_data,
                },
                occurred_at=occurred_at,
            )
            return _price, created, None
        except Exception as e:
            return None, False, e

    @classmethod
    def sync_from_paddle(cls) -> tuple[int, int]:
        logger.info("Sync Prices from Paddle")
        created = 0
        updated = 0
        error = 0
        for prices in cls.api_list_prices_generator():
            for price_data in prices.data:
                _price, _created, _error = cls.from_paddle_data(price_data)
                if _error:
                    error += 1
                elif _created:
                    created += 1
                else:
                    updated += 1
            logger.info("Price sync progress --- synced: %s, created: %s, errors: %s", updated, created, error)
        return created, updated


class Discount(PaddleBaseModel):
    id = models.CharField(max_length=50, primary_key=True)
    data = models.JSONField(null=True, blank=True, encoder=PrettyJSONEncoder)
    custom_data = models.JSONField(null=True, blank=True, encoder=PrettyJSONEncoder)

    class Meta:
        pass

    def __str__(self) -> str:
        return str(self.pk)

    def get_data(self) -> discount.Discount | None:
        if self.data is None:
            return None
        return discount.Discount.model_validate(self.data)

    @classmethod
    def api_list_discounts(cls) -> discount.DiscountsResponse:
        return paddle_client.list_discounts()

    @classmethod
    def api_list_discounts_generator(cls, **kwargs) -> Iterator[discount.DiscountsResponse]:
        yield from paginate(paddle_client.list_discounts, query_params=discount.DiscountQueryParams(**kwargs))

    @classmethod
    def from_paddle_data(cls, data, occurred_at=None) -> tuple["Discount | None", bool, Exception | None]:
        try:
            _discount, created = cls.update_or_create(
                query={"pk": data.id},
                defaults={
                    "data": data.dict(),
                    "custom_data": data.custom_data,
                },
                occurred_at=occurred_at,
            )
            return _discount, created, None
        except Exception as e:
            return None, False, e

    @classmethod
    def sync_from_paddle(cls) -> tuple[int, int]:
        logger.info("Sync Discounts from Paddle")
        created = 0
        updated = 0
        error = 0
        for discounts in cls.api_list_discounts_generator():
            for discount_data in discounts.data:
                _discount, _created, _error = cls.from_paddle_data(discount_data)
                if _error:
                    error += 1
                elif _created:
                    created += 1
                else:
                    updated += 1
            logger.info("Discount sync progress --- synced: %s, created: %s, errors: %s", updated, created, error)
        return created, updated


class Customer(PaddleBaseModel):
    id = models.CharField(max_length=50, primary_key=True)
    data = models.JSONField(null=True, blank=True, encoder=PrettyJSONEncoder)
    custom_data = models.JSONField(null=True, blank=True, encoder=PrettyJSONEncoder)
    name = models.CharField(max_length=255, null=True, blank=True)
    email = models.EmailField()
    user = models.ForeignKey(
        to=UserModel,
        null=True,
        on_delete=models.SET_NULL,
        related_name="customers",
    )

    class Meta:
        pass

    def __str__(self) -> str:
        return str(self.pk)

    def get_data(self) -> customer.Customer | None:
        if self.data is None:
            return None
        return customer.Customer.model_validate(self.data)

    @classmethod
    def api_list_customers(cls) -> customer.CustomersResponse:
        return paddle_client.list_customers()

    @classmethod
    def api_list_customers_generator(cls, **kwargs) -> Iterator[customer.CustomersResponse]:
        yield from paginate(paddle_client.list_customers, query_params=customer.CustomerQueryParams(**kwargs))

    @classmethod
    def from_paddle_data(cls, data, occurred_at=None) -> tuple["Customer | None", bool, Exception | None]:
        try:
            defaults = {
                "name": data.name,
                "email": data.email,
                "data": data.dict(),
                "custom_data": data.custom_data,
            }
            user = UserModel.objects.filter(email=data.email).first()
            if user is not None:
                defaults["user_id"] = user.pk

            instance, created = cls.update_or_create(
                query={"pk": data.id},
                defaults=defaults,
                occurred_at=occurred_at,
            )

            return instance, created, None
        except Exception as e:
            return None, False, e

    @classmethod
    def sync_from_paddle(cls, include_addresses=True, include_businesses=True, include_subscriptions=True) -> None:
        logger.info("Sync Customers from Paddle")
        created = 0
        updated = 0
        error = 0
        for customers in cls.api_list_customers_generator():
            for customer_data in customers.data:
                _customer, created, _error = cls.from_paddle_data(customer_data)
                if _customer:
                    if include_addresses:
                        _customer.sync_addresses_from_paddle()
                    if include_businesses:
                        _customer.sync_businesses_from_paddle()
                    if include_subscriptions:
                        _customer.sync_subscription_from_paddle()
                if _error:
                    error += 1
                elif created:
                    created += 1
                else:
                    updated += 1
            logger.info("Customer sync progress --- synced: %s, created: %s, error: %s", updated, created, error)

    def sync_addresses_from_paddle(self) -> None:
        logger.info("Address sync from paddle for customer: %s", self.pk)
        count = 0
        for addresses in Address.api_list_addresses_for_customer_generator(customer_id=self.pk):
            for address_data in addresses.data:
                Address.from_paddle_data(address_data, self.pk)
                count += 1
            logger.info("Address sync progress --- synced: %s", count)

    def sync_businesses_from_paddle(self) -> None:
        logger.info("Business sync from paddle for customer: %s", self.pk)
        count = 0
        for businesses in Business.api_list_businesses_for_customer_generator(customer_id=self.pk):
            for business_data in businesses.data:
                Business.from_paddle_data(business_data, self.pk)
                count += 1
            logger.info("Business sync progress --- synced: %s", count)

    def sync_subscription_from_paddle(self) -> None:
        logger.info("Subscription sync from paddle for customer: %s", self.pk)
        count = 0
        for subscriptions in Subscription.api_list_subscriptions_generator(customer_id=self.pk):
            for subscription_data in subscriptions.data:
                Subscription.from_paddle_data(subscription_data)
                count += 1
            logger.info("Subscription sync progress --- synced: %s", count)


class Address(PaddleBaseModel):
    id = models.CharField(max_length=50, primary_key=True)
    customer = models.ForeignKey(Customer, on_delete=models.CASCADE, related_name="addresses", null=True, blank=True)
    data = models.JSONField(null=True, blank=True, encoder=PrettyJSONEncoder)
    custom_data = models.JSONField(null=True, blank=True, encoder=PrettyJSONEncoder)
    country_code = models.CharField(max_length=2)

    class Meta:
        pass

    def __str__(self) -> str:
        return str(self.pk)

    def get_data(self) -> address.Address | None:
        if self.data is None:
            return None
        return address.Address.model_validate(self.data)

    @classmethod
    def api_list_addresses_for_customer(cls, customer_id) -> address.AddressesResponse:
        return paddle_client.list_addresses_for_customer(customer_id=customer_id)

    @classmethod
    def api_list_addresses_for_customer_generator(cls, customer_id, **kwargs) -> Iterator[address.AddressesResponse]:
        yield from paginate(
            paddle_client.list_addresses_for_customer,
            customer_id=customer_id,
            query_params=address.AddressQueryParams(**kwargs),
        )

    @classmethod
    def from_paddle_data(
        cls, data, customer_id=None, occurred_at=None
    ) -> tuple["Address | None", bool, Exception | None]:
        try:
            defaults = {
                "data": data.dict(),
                "custom_data": data.custom_data,
                "country_code": data.country_code,
            }
            if customer_id is not None:
                defaults["customer_id"] = customer_id
            elif data.customer_id is not None:
                defaults["customer_id"] = data.customer_id
            _address, created = cls.update_or_create(
                query={"pk": data.id},
                defaults=defaults,
                occurred_at=occurred_at,
            )
            return _address, created, None
        except Exception as e:
            return None, False, e

    @classmethod
    def sync_from_paddle(cls) -> None:
        logger.info("Address sync from paddle")
        customers = Customer.objects.all()
        for _customer in customers:
            for addresses in cls.api_list_addresses_for_customer_generator(customer_id=_customer.pk):
                for address_data in addresses.data:
                    cls.from_paddle_data(address_data, customer_id=_customer.pk)


class Business(PaddleBaseModel):
    id = models.CharField(max_length=50, primary_key=True)
    customer = models.ForeignKey(Customer, on_delete=models.CASCADE, related_name="businesses", null=True, blank=True)
    data = models.JSONField(null=True, blank=True, encoder=PrettyJSONEncoder)
    custom_data = models.JSONField(null=True, blank=True, encoder=PrettyJSONEncoder)

    class Meta:
        pass

    def __str__(self) -> str:
        return str(self.pk)

    def get_data(self) -> business.Business | None:
        if self.data is None:
            return None
        return business.Business.model_validate(self.data)

    @classmethod
    def api_list_businesses_for_customer(cls, customer_id) -> business.BusinessesResponse:
        return paddle_client.list_businesses_for_customer(customer_id=customer_id)

    @classmethod
    def api_list_businesses_for_customer_generator(cls, customer_id, **kwargs) -> Iterator[business.BusinessesResponse]:
        yield from paginate(
            paddle_client.list_businesses_for_customer,
            customer_id=customer_id,
            query_params=business.BusinessQueryParams(**kwargs),
        )

    @classmethod
    def from_paddle_data(cls, data, customer_id, occurred_at=None) -> tuple["Business | None", bool, Exception | None]:
        try:
            defaults = {
                "data": data.dict(),
                "custom_data": data.custom_data,
            }
            if customer_id is not None:
                defaults["customer_id"] = customer_id
            _business, created = cls.update_or_create(
                query={"pk": data.id},
                defaults=defaults,
                occurred_at=occurred_at,
            )
            return _business, created, None
        except Exception as e:
            return None, False, e

    @classmethod
    def sync_from_paddle(cls) -> None:
        logger.info("Business sync from paddle")
        customers = Customer.objects.all()
        for _customer in customers:
            for businesses in cls.api_list_businesses_for_customer_generator(customer_id=_customer.pk):
                for business_data in businesses.data:
                    cls.from_paddle_data(business_data, customer_id=_customer.pk)


class Subscription(PaddleBaseModel):
    id = models.CharField(max_length=50, primary_key=True)
    data = models.JSONField(null=True, blank=True, encoder=PrettyJSONEncoder)
    custom_data = models.JSONField(null=True, blank=True, encoder=PrettyJSONEncoder)
    account = models.ForeignKey(
        to=get_account_model(),
        null=True,
        on_delete=models.SET_NULL,
        related_name="subscriptions",
    )
    customer = models.ForeignKey(Customer, on_delete=models.CASCADE, related_name="subscriptions")
    address = models.ForeignKey(
        "Address", on_delete=models.CASCADE, null=True, blank=True, related_name="subscriptions"
    )
    business = models.ForeignKey(
        "Business", on_delete=models.CASCADE, null=True, blank=True, related_name="subscriptions"
    )
    status = models.CharField(
        max_length=10,
        choices=[
            ("active", "Active"),
            ("canceled", "Canceled"),
            ("past_due", "Past Due"),
            ("paused", "Paused"),
            ("trialing", "Trialing"),
        ],
    )
    products = models.ManyToManyField(Product, related_name="subscriptions")

    class Meta:
        pass

    def __str__(self) -> str:
        return str(self.pk)

    def get_data(self) -> subscription.Subscription | None:
        if self.data is None:
            return None
        return subscription.Subscription.model_validate(self.data)

    @classmethod
    def api_list_subscriptions(cls) -> subscription.SubscriptionsResponse:
        return paddle_client.list_subscriptions()

    @classmethod
    def api_list_subscriptions_generator(cls, **kwargs) -> Iterator[subscription.SubscriptionsResponse]:
        yield from paginate(
            paddle_client.list_subscriptions, query_params=subscription.SubscriptionQueryParams(**kwargs)
        )

    @classmethod
    def api_get_subscription(cls, subscription_id) -> subscription.SubscriptionResponse:
        return paddle_client.get_subscription(subscription_id)

    @classmethod
    def from_paddle_data(cls, data, occurred_at=None) -> tuple["Subscription | None", bool, Exception | str | None]:
        account_id = None
        try:
            account_id = data.custom_data["account_id"]
        except (KeyError, TypeError):
            pass

        if account_id is not None:
            if not get_account_model().objects.filter(pk=account_id).exists():
                error = f"Subscription: Account with id: {account_id} does not exist"
                return None, False, error

        try:
            defaults = {
                "customer_id": data.customer_id,
                "address_id": data.address_id,
                "business_id": data.business_id,
                "status": data.status,
                "data": data.dict(),
                "custom_data": data.custom_data,
            }
            if account_id is not None:
                defaults["account_id"] = account_id

            if settings.PADDLE_ACCOUNT_LINK_BY_CUSTOMER and settings.PADDLE_ACCOUNT_MODEL == django_settings.AUTH_USER_MODEL:
                # The Account is the User model, link by email is safe
                if "email" in data.custom_data:
                    email = data.custom_data["email"]
                    _user = UserModel.objects.filter(email=email).first()
                    if _user is not None:
                        defaults["account_id"] = _user.id
                    else:
                        logger.warning(
                            "Cannot link account for subscription %s, user not found for email %s", data.id, email
                        )
                else:
                    # Link by customer.email via customer_id
                    _customer = Customer.objects.get(pk=data.customer_id)
                    if _customer.user is not None:
                        defaults["account_id"] = _customer.user_id
                    else:
                        logger.warning("Cannot link account for subscription %s, customer has no user", data.id)

            _subscription, created = cls.update_or_create(
                query={"pk": data.id},
                defaults=defaults,
                occurred_at=occurred_at,
            )
            product_ids = [item.price.product_id for item in data.items]
            _subscription.products.set(product_ids)
            return _subscription, created, None
        except Exception as e:
            return None, False, e

    @classmethod
    def sync_from_paddle(cls, **kwargs) -> tuple[int, int]:
        logger.info("Sync Subscriptions from Paddle")
        created = 0
        updated = 0
        error = 0
        for subscriptions in cls.api_list_subscriptions_generator(**kwargs):
            for subscription_data in subscriptions.data:
                _subscription, _created, _error = cls.from_paddle_data(subscription_data)
                if _error:
                    error += 1
                elif _created:
                    created += 1
                else:
                    updated += 1
            logger.info("Subscription sync progress --- synced: %s, created: %s, error: %s", updated, created, error)
        return created, updated


class Transaction(PaddleBaseModel):
    id = models.CharField(max_length=50, primary_key=True)
    data = models.JSONField(null=True, blank=True, encoder=PrettyJSONEncoder)
    custom_data = models.JSONField(null=True, blank=True, encoder=PrettyJSONEncoder)
    customer = models.ForeignKey(Customer, on_delete=models.CASCADE, related_name="transactions")
    subscription = models.ForeignKey(
        Subscription, on_delete=models.CASCADE, related_name="transactions", null=True, blank=True
    )

    class Meta:
        pass

    def __str__(self) -> str:
        return str(self.pk)

    def get_data(self) -> transaction.Transaction | None:
        if self.data is None:
            return None
        return transaction.Transaction.model_validate(self.data)

    @classmethod
    def api_list_transactions(cls) -> transaction.TransactionsResponse:
        return paddle_client.list_transactions()

    @classmethod
    def api_list_transactions_generator(cls, **kwargs) -> Iterator[transaction.TransactionsResponse]:
        yield from paginate(paddle_client.list_transactions, query_params=transaction.TransactionQueryParams(**kwargs))

    @classmethod
    def from_paddle_data(cls, data, occurred_at=None) -> tuple["Transaction | None", bool, Exception | None]:
        try:
            _transaction, created = cls.update_or_create(
                query={"pk": data.id},
                defaults={
                    "customer_id": data.customer_id,
                    "subscription_id": data.subscription_id,
                    "data": data.dict(),
                    "custom_data": data.custom_data,
                },
                occurred_at=occurred_at,
            )
            return _transaction, created, None
        except Exception as e:
            logger.info(e)
            return None, False, e

    @classmethod
    def sync_from_paddle(cls) -> tuple[int, int]:
        logger.info("Sync Transactions from Paddle")
        created = 0
        updated = 0
        error = 0
        for transactions in cls.api_list_transactions_generator():
            for transaction_data in transactions.data:
                _transaction, _created, _error = cls.from_paddle_data(transaction_data)
                if _error:
                    error += 1
                elif _created:
                    created += 1
                else:
                    updated += 1
            logger.info("Transaction sync progress --- synced: %s, created: %s, error: %s", updated, created, error)
        return created, updated

    @classmethod
    def sync_from_paddle_for_subscription(cls, subscription_id) -> tuple[int, int]:
        logger.info("Sync Transactions from Paddle for subscription: %s", subscription_id)
        created = 0
        updated = 0
        error = 0
        for transactions in cls.api_list_transactions_generator(subscription_id=subscription_id):
            for transaction_data in transactions.data:
                if transaction_data.subscription_id == subscription_id:
                    _transaction, _created, _error = cls.from_paddle_data(transaction_data)
                    if _error:
                        error += 1
                    elif _created:
                        created += 1
                    else:
                        updated += 1
            logger.info("Transaction sync progress --- synced: %s, created: %s, error: %s", updated, created, error)
        return created, updated


@receiver(signals.address_created)
@receiver(signals.address_imported)
@receiver(signals.address_updated)
def address_event_handler(sender, payload, *args, **kwargs) -> None:
    if not isinstance(payload, address.Address):
        payload = address.Address.model_validate(payload)
    occurred_at = kwargs.get("occurred_at")

    _, _, error = Address.from_paddle_data(payload, None, occurred_at)
    if error:
        raise DjangoPaddleBillingError(error)


@receiver(signals.adjustment_created)
@receiver(signals.adjustment_updated)
def adjustment_event_handler(sender, payload, *args, **kwargs) -> None:
    pass


@receiver(signals.business_created)
@receiver(signals.business_imported)
@receiver(signals.business_updated)
def business_event_handler(sender, payload, *args, **kwargs) -> None:
    if not isinstance(payload, business.Business):
        payload = business.Business.model_validate(payload)
    occurred_at = kwargs.get("occurred_at")

    _, _, error = Business.from_paddle_data(payload, None, occurred_at)
    if error:
        raise DjangoPaddleBillingError(error)


@receiver(signals.customer_created)
@receiver(signals.customer_imported)
@receiver(signals.customer_updated)
def customer_event_handler(sender, payload, *args, **kwargs) -> None:
    if not isinstance(payload, customer.Customer):
        payload = customer.Customer.model_validate(payload)
    occurred_at = kwargs.get("occurred_at")

    _, _, error = Customer.from_paddle_data(payload, occurred_at)
    if error:
        raise DjangoPaddleBillingError(error)


@receiver(signals.discount_created)
@receiver(signals.discount_imported)
@receiver(signals.discount_updated)
def discount_event_handler(sender, payload, *args, **kwargs) -> None:
    if not isinstance(payload, discount.Discount):
        payload = discount.Discount.model_validate(payload)
    occurred_at = kwargs.get("occurred_at")

    _, _, error = Discount.from_paddle_data(payload, occurred_at)
    if error:
        raise DjangoPaddleBillingError(error)


@receiver(signals.payout_created)
@receiver(signals.payout_paid)
def payout_event_handler(sender, payload, *args, **kwargs) -> None:
    pass


@receiver(signals.price_created)
@receiver(signals.price_imported)
@receiver(signals.price_updated)
def price_event_handler(sender, payload, *args, **kwargs) -> None:
    if not isinstance(payload, price.Price):
        payload = price.Price.model_validate(payload)
    occurred_at = kwargs.get("occurred_at")

    _, _, error = Price.from_paddle_data(payload, occurred_at)
    if error:
        raise DjangoPaddleBillingError(error)


@receiver(signals.product_created)
@receiver(signals.product_imported)
@receiver(signals.product_updated)
def product_event_handler(sender, payload, *args, **kwargs) -> None:
    if not isinstance(payload, product.Product):
        payload = product.Product.model_validate(payload)
    occurred_at = kwargs.get("occurred_at")

    _, _, error = Product.from_paddle_data(payload, occurred_at)
    if error:
        raise DjangoPaddleBillingError(error)


@receiver(signals.report_created)
@receiver(signals.report_updated)
def report_event_handler(sender, payload, *args, **kwargs) -> None:
    pass


@receiver(signals.subscription_activated)
@receiver(signals.subscription_canceled)
@receiver(signals.subscription_created)
@receiver(signals.subscription_imported)
@receiver(signals.subscription_past_due)
@receiver(signals.subscription_paused)
@receiver(signals.subscription_resumed)
@receiver(signals.subscription_trialing)
@receiver(signals.subscription_updated)
def subscription_event_handler(sender, payload, *args, **kwargs) -> None:
    if not isinstance(payload, subscription.Subscription):
        payload = subscription.Subscription.model_validate(payload)
    occurred_at = kwargs.get("occurred_at")

    _, _, error = Subscription.from_paddle_data(payload, occurred_at)
    if error:
        raise DjangoPaddleBillingError(error)


@receiver(signals.transaction_billed)
@receiver(signals.transaction_cancelled)
@receiver(signals.transaction_completed)
@receiver(signals.transaction_created)
@receiver(signals.transaction_paid)
@receiver(signals.transaction_past_due)
@receiver(signals.transaction_payment_failed)
@receiver(signals.transaction_ready)
@receiver(signals.transaction_updated)
def transaction_event_handler(sender, payload, *args, **kwargs) -> None:
    if not isinstance(payload, transaction.Transaction):
        payload = transaction.Transaction.model_validate(payload)
    occurred_at = kwargs.get("occurred_at")

    _, _, error = Transaction.from_paddle_data(payload, occurred_at)
    if error:
        raise DjangoPaddleBillingError(error)



# Generated by Django 5.0.4 on 2024-06-01 08:46

from django.db import migrations, models

import django_paddle_billing.encoders


class Migration(migrations.Migration):

    dependencies = [
        ("django_paddle_billing", "0002_alter_subscription_status"),
    ]

    operations = [
        migrations.CreateModel(
            name="Discount",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("occurred_at", models.DateTimeField(blank=True, null=True)),
                ("id", models.CharField(max_length=50, primary_key=True, serialize=False)),
                (
                    "data",
                    models.JSONField(blank=True, encoder=django_paddle_billing.encoders.PrettyJSONEncoder, null=True),
                ),
                (
                    "custom_data",
                    models.JSONField(blank=True, encoder=django_paddle_billing.encoders.PrettyJSONEncoder, null=True),
                ),
            ],
        ),
    ]



from http import HTTPStatus

from billing.views import router as billing_router
from django.core.exceptions import (
    FieldError,
    ObjectDoesNotExist,
    PermissionDenied,
    ValidationError,
)
from ninja import NinjaAPI
from ninja.errors import ValidationError as NinjaValidationError

api = NinjaAPI(urls_namespace="api")

api.add_router(router=billing_router, prefix="/billing")


@api.exception_handler(ObjectDoesNotExist)
def handle_object_does_not_exist(request, exc):
    return api.create_response(
        request,
        {"message": "ObjectDoesNotExist", "detail": str(exc)},
        status=HTTPStatus.NOT_FOUND,
    )


@api.exception_handler(PermissionDenied)
def handle_permission_error(request, exc: PermissionDenied):
    return api.create_response(
        request,
        {
            "message": "PermissionDenied",
            "detail": "You don't have the permission to access this resource.",
        },
        status=HTTPStatus.FORBIDDEN,
    )


@api.exception_handler(NinjaValidationError)
def handle_ninja_validation_error(request, exc: NinjaValidationError):
    mapped_msg = {error["loc"][-1]: error["msg"] for error in exc.errors}
    return api.create_response(
        request,
        data={"message": "NinjaValidationError", "detail": mapped_msg},
        status=HTTPStatus.BAD_REQUEST,
    )


@api.exception_handler(ValidationError)
def handle_validation_error(request, exc: ValidationError):
    status = HTTPStatus.BAD_REQUEST
    for field, errors in exc.error_dict.items():
        for error in errors:
            if error.code in ["unique", "unique_together"]:
                status = HTTPStatus.CONFLICT
    return api.create_response(
        request,
        data={"message": "ValidationError", "detail": exc.message_dict},
        status=status,
    )


@api.exception_handler(FieldError)
def handle_field_error(request, exc: FieldError):
    return api.create_response(
        request,
        data={"message": "FieldError", "detail": str(exc)},
        status=HTTPStatus.BAD_REQUEST,
    )



import django.dispatch

# Address Alerts
address_created = django.dispatch.Signal()
address_imported = django.dispatch.Signal()
address_updated = django.dispatch.Signal()

# Adjustment Alerts
adjustment_created = django.dispatch.Signal()
adjustment_updated = django.dispatch.Signal()

# Business Alerts
business_created = django.dispatch.Signal()
business_imported = django.dispatch.Signal()
business_updated = django.dispatch.Signal()

# Customer Alerts
customer_created = django.dispatch.Signal()
customer_imported = django.dispatch.Signal()
customer_updated = django.dispatch.Signal()

# Discount Alerts
discount_created = django.dispatch.Signal()
discount_imported = django.dispatch.Signal()
discount_updated = django.dispatch.Signal()

# Payout Alerts  # Payment by Paddle for me (Company)
payout_created = django.dispatch.Signal()
payout_paid = django.dispatch.Signal()

# Price Alerts
price_created = django.dispatch.Signal()
price_imported = django.dispatch.Signal()
price_updated = django.dispatch.Signal()

# Product Alerts
product_created = django.dispatch.Signal()
product_imported = django.dispatch.Signal()
product_updated = django.dispatch.Signal()

# Report Alerts
report_created = django.dispatch.Signal()
report_updated = django.dispatch.Signal()

# Subscription Alerts
subscription_activated = django.dispatch.Signal()
subscription_canceled = django.dispatch.Signal()
subscription_created = django.dispatch.Signal()
subscription_imported = django.dispatch.Signal()
subscription_past_due = django.dispatch.Signal()
subscription_paused = django.dispatch.Signal()
subscription_resumed = django.dispatch.Signal()
subscription_trialing = django.dispatch.Signal()
subscription_updated = django.dispatch.Signal()

# Transaction Alerts
transaction_billed = django.dispatch.Signal()
transaction_cancelled = django.dispatch.Signal()
transaction_completed = django.dispatch.Signal()
transaction_created = django.dispatch.Signal()
transaction_paid = django.dispatch.Signal()
transaction_past_due = django.dispatch.Signal()
transaction_payment_failed = django.dispatch.Signal()
transaction_ready = django.dispatch.Signal()
transaction_updated = django.dispatch.Signal()



# Generated by Django 5.0.1 on 2024-05-22 07:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("django_paddle_billing", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="subscription",
            name="status",
            field=models.CharField(
                choices=[
                    ("active", "Active"),
                    ("canceled", "Canceled"),
                    ("past_due", "Past Due"),
                    ("paused", "Paused"),
                    ("trialing", "Trialing"),
                ],
                max_length=10,
            ),
        ),
    ]



from django.core.management.base import BaseCommand


class Command(BaseCommand):
    help = "Sync data from Paddle"

    def handle(self, *args, **options):
        try:
            # ----------------
            # Address
            from django_paddle_billing.models import Address

            Address.sync_from_paddle()

            # ----------------
            # Business
            from django_paddle_billing.models import Business

            Business.sync_from_paddle()

            # ----------------
            # Products
            from django_paddle_billing.models import Product

            Product.sync_from_paddle()
            # products = Product.objects.all()

            # ----------------
            # Prices
            from django_paddle_billing.models import Price

            Price.sync_from_paddle()

            # ----------------
            # Discounts
            from django_paddle_billing.models import Discount

            Discount.sync_from_paddle()

            # ----------------
            # Customers
            from django_paddle_billing.models import Customer

            Customer.sync_from_paddle()

            # ----------------
            # Subscriptions
            from django_paddle_billing.models import Subscription

            Subscription.sync_from_paddle()

            # ----------------
            # Transactions
            from django_paddle_billing.models import Transaction

            Transaction.sync_from_paddle()

            self.stdout.write(self.style.SUCCESS("Successfully synced data from Paddle"))

        except Exception as e:
            self.stdout.write(self.style.ERROR("Failed to sync data from Paddle"))
            self.stdout.write(self.style.ERROR(e))



from ninja.router import Router

from django_paddle_billing.models import Product, Subscription

from .schema import ProductSubscriptionSchema

router = Router()


@router.get("/products", response=ProductSubscriptionSchema)
def products(request):
    products = Product.objects.values("id", "name", "created_at", "updated_at", "status")
    subscriptions = Subscription.objects.filter(account_id=1).prefetch_related("products").all()
    result = {"products": list(products), "subscriptions": list(subscriptions)}

    return result



from django.apps import AppConfig


class DjangoPaddleBillingConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "django_paddle_billing"
    verbose_name = "Paddle Billing"



import typing

from django.conf import settings
from django.contrib import admin
from django.db import models

from django_paddle_billing import settings as app_settings
from django_paddle_billing.models import (
    Address,
    Business,
    Customer,
    Discount,
    Price,
    Product,
    Subscription,
    Transaction,
)

# Check if unfold is in installed apps
if "unfold" in settings.INSTALLED_APPS:
    from unfold.admin import ModelAdmin, StackedInline, TabularInline
else:
    from django.contrib.admin import ModelAdmin, StackedInline, TabularInline


class AddressInline(StackedInline):
    model = Address
    extra = 1


class BusinessInline(StackedInline):
    model = Business
    extra = 1


class PriceInline(StackedInline):
    model = Price
    extra = 1


class ProductInline(TabularInline):
    model = Product.subscriptions.through
    extra = 1
    show_change_link = True


class CustomerInline(StackedInline):
    model = Customer
    extra = 1


class SubscriptionInline(StackedInline):
    model = Subscription
    extra = 1


class TransactionInline(TabularInline):
    model = Transaction
    extra = 1
    show_change_link = True


@admin.register(Address)
class AddressAdmin(ModelAdmin):
    list_display = ["customer_email", "country_code", "postal_code", "status"]
    inlines = (SubscriptionInline,)
    formfield_overrides: typing.ClassVar = {
        models.JSONField: {"widget": app_settings.ADMIN_JSON_EDITOR_WIDGET},
    }

    def has_change_permission(self, request, obj=None):
        return not app_settings.ADMIN_READONLY

    def customer_email(self, obj=None):
        if obj and obj.customer:
            return obj.customer.email
        if obj:
            return obj.id
        return ""

    def postal_code(self, obj=None):
        if obj and obj.data:
            return obj.data.get("postal_code", "")
        return ""

    def status(self, obj=None):
        if obj and obj.data:
            return obj.data.get("status", "")
        return ""


@admin.register(Business)
class BusinessAdmin(ModelAdmin):
    list_display = [
        "name",
        "company_number",
        "tax_identifier",
        "status",
    ]
    inlines = (SubscriptionInline,)
    formfield_overrides: typing.ClassVar = {
        models.JSONField: {"widget": app_settings.ADMIN_JSON_EDITOR_WIDGET},
    }

    def has_change_permission(self, request, obj=None):
        return not app_settings.ADMIN_READONLY

    def name(self, obj=None):
        if obj and obj.data:
            return obj.data.get("name", obj.id)
        if obj:
            return obj.id
        return ""

    def company_number(self, obj=None):
        if obj and obj.data:
            return obj.data.get("company_number", "")
        return ""

    def tax_identifier(self, obj=None):
        if obj and obj.data:
            return obj.data.get("tax_identifier", "")
        return ""

    def status(self, obj=None):
        if obj and obj.data:
            return obj.data.get("status", "")
        return ""


@admin.register(Product)
class ProductAdmin(ModelAdmin):
    list_display = [
        "name",
        "status",
        "created_at",
    ]
    search_fields = ["id", "name"]
    inlines = (PriceInline,)
    formfield_overrides: typing.ClassVar = {
        models.JSONField: {"widget": app_settings.ADMIN_JSON_EDITOR_WIDGET},
    }

    def has_change_permission(self, request, obj=None):
        return not app_settings.ADMIN_READONLY


@admin.register(Price)
class PriceAdmin(ModelAdmin):
    list_display = [
        "name",
        "unit_price",
        "description",
        "status",
        "trial_period",
        "billing_cycle",
    ]
    formfield_overrides: typing.ClassVar = {
        models.JSONField: {"widget": app_settings.ADMIN_JSON_EDITOR_WIDGET},
    }

    def has_change_permission(self, request, obj=None):
        return not app_settings.ADMIN_READONLY

    def billing_cycle(self, obj=None):
        if obj and obj.data and obj.data.get("billing_cycle"):
            return f'{obj.data["billing_cycle"]["frequency"]} {obj.data["billing_cycle"]["interval"]}'
        return ""

    def description(self, obj=None):
        if obj and obj.data:
            return obj.data.get("description", "")

    def name(self, obj=None):
        if obj and obj.data:
            return obj.data.get("name", obj.id)
        if obj:
            return obj.id
        return ""

    def status(self, obj=None):
        if obj and obj.data:
            return obj.data.get("status", "")

    def trial_period(self, obj=None):
        if obj and obj.data and obj.data.get("trial_period"):
            return f'{obj.data["trial_period"]["frequency"]} {obj.data["trial_period"]["interval"]}'
        return ""

    def unit_price(self, obj=None):
        if obj and obj.data and obj.data.get("unit_price"):
            return f'{int(obj.data["unit_price"]["amount"]) / 100} {obj.data["unit_price"]["currency_code"]}'
        return ""


@admin.register(Discount)
class DiscountAdmin(ModelAdmin):
    list_display = [
        "discount_description",
        "amount",
        "applies_to",
        "status",
        "discount_code",
        "uses_left",
        "expires",
    ]
    formfield_overrides: typing.ClassVar = {
        models.JSONField: {"widget": app_settings.ADMIN_JSON_EDITOR_WIDGET},
    }

    def has_change_permission(self, request, obj=None):
        return not app_settings.ADMIN_READONLY

    def discount_description(self, obj=None):
        if obj and obj.data:
            return obj.data.get("description", obj.id)
        if obj:
            return obj.id
        return ""

    def amount(self, obj=None):
        if obj and obj.data:
            if obj.data.get("type") == "percentage":
                return f"{obj.data.get('amount', '')}%"
            return f'{obj.data.get("amount", "")} {obj.data.get("currency_code", "")}'
        return ""

    def applies_to(self, obj=None):
        if obj and obj.data:
            return obj.data.get("restrict_to", "")
        return ""

    def status(self, obj=None):
        if obj and obj.data:
            return obj.data.get("status", "")
        return ""

    def discount_code(self, obj=None):
        if obj and obj.data:
            return obj.data.get("code", "")
        return ""

    def uses_left(self, obj=None):
        if obj and obj.data:
            return obj.data.get("usage_limit", "")
        return ""

    def expires(self, obj=None):
        if obj and obj.data:
            return obj.data.get("expires_at", "")
        return ""


@admin.register(Subscription)
class SubscriptionAdmin(ModelAdmin):
    list_display = [
        "customer_email",
        "name",
        "price",
        "next_payment",
        "status",
    ]
    inlines = (
        TransactionInline,
        ProductInline,
    )
    formfield_overrides: typing.ClassVar = {
        models.JSONField: {"widget": app_settings.ADMIN_JSON_EDITOR_WIDGET},
    }

    exclude = ["products"]

    def has_change_permission(self, request, obj=None):
        return not app_settings.ADMIN_READONLY

    def customer_email(self, obj=None):
        if obj and obj.customer:
            return obj.customer.email
        if obj:
            return obj.id
        return ""

    def name(self, obj=None):
        if obj and obj.data:
            try:
                return ", ".join([item["price"]["name"] for item in obj.data["items"]])
            except Exception:
                return ""
        return ""

    def price(self, obj=None):
        if obj and obj.data:
            try:
                unit_price = [int(item["price"]["unit_price"]["amount"]) / 100 for item in obj.data["items"]]
                frequency = [item["price"]["billing_cycle"] for item in obj.data["items"]]
                return ", ".join(
                    [
                        f"{unit_price[i]}/{frequency[i]['frequency']} {frequency[i]['interval']}"
                        for i in range(len(unit_price))
                    ]
                )
            except Exception:
                return ""
        return ""

    def next_payment(self, obj=None):
        if obj and obj.data:
            try:
                return obj.data["next_billed_at"]
            except Exception:
                return ""
        return ""


@admin.register(Customer)
class CustomerAdmin(ModelAdmin):
    list_display = [
        "email",
        "name",
        "status",
        "created_at",
    ]
    inlines = (
        AddressInline,
        BusinessInline,
        SubscriptionInline,
        TransactionInline,
    )
    formfield_overrides: typing.ClassVar = {
        models.JSONField: {"widget": app_settings.ADMIN_JSON_EDITOR_WIDGET},
    }

    def has_change_permission(self, request, obj=None):
        return not app_settings.ADMIN_READONLY

    def status(self, obj=None):
        if obj and obj.data:
            return obj.data.get("status", "")


@admin.register(Transaction)
class TransactionAdmin(ModelAdmin):
    list_display = ["customer_email", "payment_amount", "payment_method", "date_paid", "products", "status"]
    formfield_overrides: typing.ClassVar = {
        models.JSONField: {"widget": app_settings.ADMIN_JSON_EDITOR_WIDGET},
    }

    def has_change_permission(self, request, obj=None):
        return not app_settings.ADMIN_READONLY

    def customer_email(self, obj=None):
        if obj and obj.customer:
            return obj.customer.email
        if obj:
            return obj.id
        return ""

    def payment_amount(self, obj=None):
        if obj and obj.data:
            try:
                return int(obj.data["details"]["totals"]["total"]) / 100
            except Exception:
                return ""
        return ""

    def payment_method(self, obj=None):
        if obj and obj.data:
            try:
                return (
                    f'{obj.data["payments"][0]["method_details"]["card"]["type"]} '
                    f'{obj.data["payments"][0]["method_details"]["card"]["last4"]}'
                )
            except Exception:
                return ""
        return ""

    def date_paid(self, obj=None):
        if obj and obj.data:
            try:
                return obj.data["payments"][0]["captured_at"]
            except Exception:
                return ""
        return ""

    def products(self, obj=None):
        if obj and obj.data:
            items = obj.data.get("items", [])
            # return items
            products = Product.objects.filter(id__in=[item["price"]["product_id"] for item in items])
            return ", ".join([product.name for product in products])
        return ""

    products.short_description = "Product(s)"

    def status(self, obj=None):
        if obj and obj.data:
            return obj.data.get("status", "")
        return ""



from ninja import ModelSchema, Schema

from django_paddle_billing.models import Product, Subscription


class SubscriptionSchema(ModelSchema):
    class Meta:
        model = Subscription
        fields = "__all__"
        # exclude = ("id",)


class ProductSchema(ModelSchema):
    # subscriptions: list[SubscriptionSchema] = []

    class Meta:
        model = Product
        fields = "__all__"
        # exclude = ("id",)


class ProductSubscriptionSchema(Schema):
    products: list[ProductSchema] = []
    subscriptions: list[SubscriptionSchema] = []



import json

from django_paddle_billing.encoders import PrettyJSONEncoder


def test_pretty_json_encoder_produces_readable_output():
    data = {"key": "value", "number": 123, "list": [1, 2, 3]}
    encoder = PrettyJSONEncoder(indent=4, sort_keys=True)
    encoded = encoder.encode(data)
    expected = json.dumps(data, indent=4, sort_keys=True)
    assert encoded == expected


def test_pretty_json_encoder_handles_empty_input():
    data = {}
    encoder = PrettyJSONEncoder(indent=4, sort_keys=True)
    encoded = encoder.encode(data)
    expected = json.dumps(data, indent=4, sort_keys=True)
    assert encoded == expected


def test_pretty_json_encoder_handles_nested_input():
    data = {"key": {"nested_key": "nested_value"}}
    encoder = PrettyJSONEncoder(indent=4, sort_keys=True)
    encoded = encoder.encode(data)
    expected = json.dumps(data, indent=4, sort_keys=True)
    assert encoded == expected



# SPDX-FileCopyrightText: 2023-present Benjamin Gervan <benjamin@websideproject.com>
#
# SPDX-License-Identifier: MIT
from django_paddle_billing.settings import settings as default_settings


class AppSettings:
    def __getattr__(self, name):
        if name in default_settings:
            return default_settings[name]

        raise AttributeError(f"'PADDLE_BILLING' settings object has no attribute '{name}'")


settings = AppSettings()



# SPDX-FileCopyrightText: 2023-present Benjamin Gervan <benjamin@websideproject.com>
#
# SPDX-License-Identifier: MIT
from django.core.serializers.json import DjangoJSONEncoder


class PrettyJSONEncoder(DjangoJSONEncoder):
    def __init__(self, *args, indent, sort_keys, **kwargs):
        super().__init__(*args, indent=4, sort_keys=True, **kwargs)



import json
import typing

from django.http import HttpResponse, HttpResponseBadRequest
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import View
from paddle_billing_client.helpers import validate_webhook_signature
from paddle_billing_client.models.notification import NotificationPayload

from django_paddle_billing import settings as app_settings
from django_paddle_billing import signals


@method_decorator(csrf_exempt, name="dispatch")
class PaddleWebhookView(View):
    SUPPORTED_WEBHOOKS: typing.ClassVar = {
        # Address Alerts
        "address.created": signals.address_created,
        "address.updated": signals.address_updated,
        "address.imported": signals.address_imported,
        # Adjustment Alerts
        "adjustment.created": signals.adjustment_created,
        "adjustment.updated": signals.adjustment_updated,
        # Business Alerts
        "business.created": signals.business_created,
        "business.imported": signals.business_imported,
        "business.updated": signals.business_updated,
        # Customer Alerts
        "customer.created": signals.customer_created,
        "customer.imported": signals.customer_imported,
        "customer.updated": signals.customer_updated,
        # Discount Alerts
        "discount.created": signals.discount_created,
        "discount.imported": signals.discount_imported,
        "discount.updated": signals.discount_updated,
        # Payout Alerts  # Payment by Paddle for me (Company)
        "payout.created": signals.payout_created,
        "payout.paid": signals.payout_paid,
        # Price Alerts
        "price.created": signals.price_created,
        "price.imported": signals.price_imported,
        "price.updated": signals.price_updated,
        # Product Alerts
        "product.created": signals.product_created,
        "product.imported": signals.product_imported,
        "product.updated": signals.product_updated,
        # Report Alerts
        "report.created": signals.report_created,
        "report.updated": signals.report_updated,
        # Subscription Alerts
        "subscription.activated": signals.subscription_activated,
        "subscription.canceled": signals.subscription_canceled,
        "subscription.created": signals.subscription_created,
        "subscription.imported": signals.subscription_imported,
        "subscription.past_due": signals.subscription_past_due,
        "subscription.paused": signals.subscription_paused,
        "subscription.resumed": signals.subscription_resumed,
        "subscription.trialing": signals.subscription_trialing,
        "subscription.updated": signals.subscription_updated,
        # Transaction Alerts
        "transaction.billed": signals.transaction_billed,
        "transaction.cancelled": signals.transaction_cancelled,
        "transaction.completed": signals.transaction_completed,
        "transaction.created": signals.transaction_created,
        "transaction.paid": signals.transaction_paid,
        "transaction.past_due": signals.transaction_past_due,
        "transaction.payment_failed": signals.transaction_payment_failed,
        "transaction.ready": signals.transaction_ready,
        "transaction.updated": signals.transaction_updated,
    }

    def post(self, request, *args, **kwargs):
        """
        handle paddle webhook requests by
        - validating the payload signature
        - sending a django signal for each of the SUPPORTED_WEBHOOKS
        """
        payload = request.body.decode("utf-8")
        paddle_ip = request.META.get(app_settings.PADDLE_IP_REQUEST_HEADER, "").split(", ")[0]
        if app_settings.PADDLE_SANDBOX and paddle_ip not in app_settings.PADDLE_SANDBOX_IPS:
            return HttpResponseBadRequest("IP not allowed")
        elif not app_settings.PADDLE_SANDBOX and paddle_ip not in app_settings.PADDLE_IPS:
            return HttpResponseBadRequest("IP not allowed")

        is_valid = validate_webhook_signature(
            request.META.get("HTTP_PADDLE_SIGNATURE", ""), request.body, app_settings.PADDLE_SECRET_KEY
        )

        if not is_valid:
            return HttpResponseBadRequest("Invalid signature")
        notification = NotificationPayload.model_validate(json.loads(payload))

        if not notification.event_type:
            return HttpResponseBadRequest("'event_type' missing")

        if notification.event_type in self.SUPPORTED_WEBHOOKS.keys():
            signal = self.SUPPORTED_WEBHOOKS.get(notification.event_type)
            if signal:  # pragma: no cover
                signal.send(sender=self.__class__, payload=notification.data, occurred_at=notification.occurred_at)

        return HttpResponse()


paddle_webhook_view = PaddleWebhookView.as_view()



# Generated by Django 4.2.3 on 2024-01-23 20:58
import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models

import django_paddle_billing.encoders
from django_paddle_billing import settings as app_settings


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        migrations.swappable_dependency(app_settings.PADDLE_ACCOUNT_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Address",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("occurred_at", models.DateTimeField(blank=True, null=True)),
                (
                    "id",
                    models.CharField(max_length=50, primary_key=True, serialize=False),
                ),
                (
                    "data",
                    models.JSONField(
                        blank=True,
                        encoder=django_paddle_billing.encoders.PrettyJSONEncoder,
                        null=True,
                    ),
                ),
                (
                    "custom_data",
                    models.JSONField(
                        blank=True,
                        encoder=django_paddle_billing.encoders.PrettyJSONEncoder,
                        null=True,
                    ),
                ),
                ("country_code", models.CharField(max_length=2)),
            ],
        ),
        migrations.CreateModel(
            name="Business",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("occurred_at", models.DateTimeField(blank=True, null=True)),
                (
                    "id",
                    models.CharField(max_length=50, primary_key=True, serialize=False),
                ),
                (
                    "data",
                    models.JSONField(
                        blank=True,
                        encoder=django_paddle_billing.encoders.PrettyJSONEncoder,
                        null=True,
                    ),
                ),
                (
                    "custom_data",
                    models.JSONField(
                        blank=True,
                        encoder=django_paddle_billing.encoders.PrettyJSONEncoder,
                        null=True,
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Customer",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("occurred_at", models.DateTimeField(blank=True, null=True)),
                (
                    "id",
                    models.CharField(max_length=50, primary_key=True, serialize=False),
                ),
                (
                    "data",
                    models.JSONField(
                        blank=True,
                        encoder=django_paddle_billing.encoders.PrettyJSONEncoder,
                        null=True,
                    ),
                ),
                (
                    "custom_data",
                    models.JSONField(
                        blank=True,
                        encoder=django_paddle_billing.encoders.PrettyJSONEncoder,
                        null=True,
                    ),
                ),
                ("name", models.CharField(blank=True, max_length=255, null=True)),
                ("email", models.EmailField(max_length=254)),
                (
                    "user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="customers",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Product",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("occurred_at", models.DateTimeField(blank=True, null=True)),
                (
                    "id",
                    models.CharField(max_length=50, primary_key=True, serialize=False),
                ),
                (
                    "data",
                    models.JSONField(
                        blank=True,
                        encoder=django_paddle_billing.encoders.PrettyJSONEncoder,
                        null=True,
                    ),
                ),
                (
                    "custom_data",
                    models.JSONField(
                        blank=True,
                        encoder=django_paddle_billing.encoders.PrettyJSONEncoder,
                        null=True,
                    ),
                ),
                ("name", models.CharField(max_length=255)),
                (
                    "status",
                    models.CharField(
                        choices=[("active", "Active"), ("archived", "Archived")],
                        max_length=10,
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Subscription",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("occurred_at", models.DateTimeField(blank=True, null=True)),
                (
                    "id",
                    models.CharField(max_length=50, primary_key=True, serialize=False),
                ),
                (
                    "data",
                    models.JSONField(
                        blank=True,
                        encoder=django_paddle_billing.encoders.PrettyJSONEncoder,
                        null=True,
                    ),
                ),
                (
                    "custom_data",
                    models.JSONField(
                        blank=True,
                        encoder=django_paddle_billing.encoders.PrettyJSONEncoder,
                        null=True,
                    ),
                ),
                (
                    "status",
                    models.CharField(
                        choices=[
                            ("active", "Active"),
                            ("trialing", "Trialing"),
                            ("paused", "Paused"),
                            ("deleted", "Deleted"),
                        ],
                        max_length=10,
                    ),
                ),
                (
                    "account",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="subscriptions",
                        to=app_settings.PADDLE_ACCOUNT_MODEL,
                    ),
                ),
                (
                    "address",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="subscriptions",
                        to="django_paddle_billing.address",
                    ),
                ),
                (
                    "business",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="subscriptions",
                        to="django_paddle_billing.business",
                    ),
                ),
                (
                    "customer",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="subscriptions",
                        to="django_paddle_billing.customer",
                    ),
                ),
                (
                    "products",
                    models.ManyToManyField(related_name="subscriptions", to="django_paddle_billing.product"),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Transaction",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("occurred_at", models.DateTimeField(blank=True, null=True)),
                (
                    "id",
                    models.CharField(max_length=50, primary_key=True, serialize=False),
                ),
                (
                    "data",
                    models.JSONField(
                        blank=True,
                        encoder=django_paddle_billing.encoders.PrettyJSONEncoder,
                        null=True,
                    ),
                ),
                (
                    "custom_data",
                    models.JSONField(
                        blank=True,
                        encoder=django_paddle_billing.encoders.PrettyJSONEncoder,
                        null=True,
                    ),
                ),
                (
                    "customer",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="transactions",
                        to="django_paddle_billing.customer",
                    ),
                ),
                (
                    "subscription",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="transactions",
                        to="django_paddle_billing.subscription",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Price",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("occurred_at", models.DateTimeField(blank=True, null=True)),
                (
                    "id",
                    models.CharField(max_length=50, primary_key=True, serialize=False),
                ),
                (
                    "data",
                    models.JSONField(
                        blank=True,
                        encoder=django_paddle_billing.encoders.PrettyJSONEncoder,
                        null=True,
                    ),
                ),
                (
                    "custom_data",
                    models.JSONField(
                        blank=True,
                        encoder=django_paddle_billing.encoders.PrettyJSONEncoder,
                        null=True,
                    ),
                ),
                (
                    "product",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="prices",
                        to="django_paddle_billing.product",
                    ),
                ),
            ],
        ),
        migrations.AddField(
            model_name="business",
            name="customer",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="businesses",
                to="django_paddle_billing.customer",
            ),
        ),
        migrations.AddField(
            model_name="address",
            name="customer",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="addresses",
                to="django_paddle_billing.customer",
            ),
        ),
    ]
